class listComprehension {
    constructor(input, list) {
        const err = this._init(input, list);
        if (err) return err;
        return this._run()
    }

    _init(input, list) {
        this.input = input.split('for');
        this.calledExpression = this._getCalledExpression(this.input);
        this.forLoop = this._getForLoop(this.input);
        this.condition = this._getIfCondition(this.input);
        this.placeHolder = this._getPlaceHolder(this.calledExpression);
        this.list = this._getInitialList(this.forLoop, list, this.condition);
        this.calledCondition = this._getCalledCondition(this.condition)
    }

    _getCalledExpression(input) {
        if (input && input.length) {
            return input[0].replace('[', '')
        } else {
            throw Error('No such called expression')
        }
    }

    _getForLoop(input) {
        if (input && input.length === 2) {
            return input[1].split('in');
        } else {
            throw Error('No such for loop');
        }
    }

    _getIfCondition(input) {
        if (input && input.length === 2) {
            return input[1].split('if');
        }
        return null;
    }

    _getPlaceHolder(calledExpression) {
        if (calledExpression) {
            const result = [];
            for (const letter of calledExpression) {
                if (this._checkIsMathOperation(letter)) break;
                result.push(letter);
            }
            return result.join('').replace(/\s/g, '');
        } else {
            throw Error('No such variable in loop');
        }
    }

    _checkIsMathOperation(letter) {
        if (letter === '+' || letter === '-' || letter === '*' || letter === '/' || letter === '%') {
            return true;
        }
    }

    _getInitialList(forLoop, list, condition) {
        if (list) {
            return list
        } else if (forLoop && forLoop.length === 2 && !condition) {
            return eval(forLoop[1].slice(0, -1));
        } else if (forLoop && forLoop.length === 2 && condition) {
            return eval(forLoop[1].split('if')[0].slice(0, -1));
        } else {
            throw Error('No initial list');
        }
    }

    _getCalledCondition(condition) {
        if (condition && condition.length === 2) {
            return condition[1].replace(']', '');
        }
        return null;
    }

    _run() {
        const results = [];
        for (const element of this.list) {
            const modifiedElement = this._modify(element, this.calledExpression);
            if (this.calledCondition) {
                const modifiedCondition = this._modify(element, this.calledCondition);
                if (modifiedCondition) {
                    results.push(modifiedElement);
                }
            } else {
                results.push(modifiedElement);
            }
        }
        return results;
    }

    _modify(element, template) {
        let replacer = this._toPrimitiveTypes(element, false);

        if (!replacer && typeof element === "object") {
            const propeties = this.placeHolder.split('.');
            propeties.shift();

            for (const property of propeties) {
                element = element[property.replace(/\s/g, '')]
            }
            replacer = this._toPrimitiveTypes(element, true)
        }
        if (!replacer) {
            throw Error(`Unprimitive type : ${typeof element}`);
        }
        const re = new RegExp(this.placeHolder, "g");
        return eval(template.replace(re, replacer));
    }

    _toPrimitiveTypes(element, isEnd) {
        if (typeof element === "string") {
            return "'" + element + "'"
        } else if (typeof element === "number") {
            return "+'" + element + "'";
        }
        if (isEnd) {
            return '"' + element + '"';
        }
    }
}

module.exports = listComprehension;

